; Terminal with cassette interface
;
; (c) K Orton 2017 All rights reserved
;
; Displays 16 lines of 25 characters on a 625 line monitor
; Characters are 8 pixels wide by 14 pixels high
;
; Port A:
;  RA0  Keyboard column 4 drive
;  RA1  Keyboard column 3 drive
;  RA2  Keyboard column 2 drive
;  RA3  Keyboard column 1 drive
;  RA4  Shift/Control ('row 9') sense
;  RA5  Acknowledgement pulse (for host writes)
;
; Port B:
;  RB0	Keyboard row 1 sense
;  RB1	Keyboard row 2 sense
;  RB2	Keyboard row 3 sense
;  RB3	Keyboard row 4 sense
;  RB4	Keyboard row 5 sense
;  RB5	Keyboard row 6 sense
;  RB6	Keyboard row 7 sense
;  RB7	Keyboard row 8 sense
;
; Port C:
;  RC0  Keyboard column 7 drive
;  RC1  Cassette input (capture 2)
;  RC2  Cassette output (pwm 1)
;  RC3  Keyboard column 6 drive
;  RC4  Keyboard column 5 drive
;  RC5  625 line sync output
;  RC6  Bleep pulse (must be NORed with RC5)
;  RC7  Pixel data (serial data)
;
; Port D: Host port data
;
; Port E: Host port control

UPPERC	EQU	1			; Make upper case the default

; PIC constants
W	EQU	0
INDF	EQU	H'0000'
TMR0	EQU	H'0001'
PCL	EQU	H'0002'
STATUS	EQU	H'0003'
FSR	EQU	H'0004'
PORTA	EQU	H'0005'
PORTB	EQU	H'0006'
PORTC	EQU	H'0007'
PORTD	EQU	H'0008'
PCLATH	EQU	H'000A'
INTCON	EQU	H'000B'
PIR1	EQU	H'000C'
PIR2	EQU	H'000D'
T1CON	EQU	H'0010'
T2CON	EQU	H'0012'
CCPR1L	EQU	H'0015'
CCP1CN	EQU	H'0017'
RCSTA	EQU	H'0018'
TXREG	EQU	H'0019'
CCPR2H	EQU	H'001C'
CCP2CN	EQU	H'001D'
OPTREG	EQU	H'0081'
TRISA	EQU	H'0085'
TRISC	EQU	H'0087'
TRISE	EQU	H'0089'
PR2	EQU	H'0092'
TXSTA	EQU	H'0098'
SPBRG	EQU	H'0099'
ADCON1	EQU	H'009F'
C	EQU	0
Z	EQU	2
RP0	EQU	5
RP1	EQU	6
IRP	EQU	7
SPEN	EQU	7
TMR2IF	EQU	1
T0IF	EQU	2
CCP2IF	EQU	0
IBF	EQU	7
CFG	EQU	H'3D32'

	__CONFIG	CFG

; Program constants
FLAGS	EQU	H'000B'		; (INTCON)
COBUF1	EQU	H'0013'		; (SSPBUF)
COBUF2	EQU	H'001E'		; (ADRESH)
CDCNT	EQU	H'008C'		; (PIE1)
CIDIV	EQU	H'008D'		; (PIE2)
SEQCNT	EQU	H'008E'		; (PCON)
PRVCAP	EQU	H'0093'		; (SSPADD)
CISR	EQU	H'009E'		; (ADRESL)
CIBUF1	EQU	H'010C'		; (EEDATA)
CIBUF2	EQU	H'010D'		; (EEADR)
VIDSTT	EQU	H'0070'
CURV	EQU	H'0071'
CURH	EQU	H'0072'
KEY1	EQU	H'0073'
MOD1	EQU	H'0074'
KEY2	EQU	H'0075'
MOD2	EQU	H'0076'
COSR	EQU	H'0077'
COCNT	EQU	H'0078'
TMP1	EQU	H'0079'
TMP2	EQU	H'007A'
TMP3	EQU	H'007B'
TMP4	EQU	H'007C'
TMP5	EQU	H'007D'
TMP6	EQU	H'007E'
TMP7	EQU	H'007F'
COL4	EQU	0
COL3	EQU	1
COL2	EQU	2
COL1	EQU	3
ROW9	EQU	4
ACKPLS	EQU	5
COL7	EQU	0
COL6	EQU	3
COL5	EQU	4
SYNC	EQU	5
BLEEP	EQU	6
STATP0	EQU	B'00000000'
STATP1	EQU	B'00100000'
STATP2	EQU	B'01000000'
STATP3	EQU	B'01100000'
CURCH	EQU	H'5F'
ESCCH	EQU	H'1B'
TABCH	EQU	H'09'
BTABCH	EQU	H'0F'
BSCH	EQU	H'08'
RETCH	EQU	H'0D'
APOCH	EQU	H'27'
SPCH	EQU	H'20'
SPCH7	EQU	H'A0'
PNDCH	EQU	H'7F'
BSLCH	EQU	H'5C'
CDTO	EQU	D'252'
ESCF	EQU	3
SHFTF	EQU	4
CTRLF	EQU	5
BELLF	EQU	6

; Delay macro (for up to 1023 cycles)
; Uses W
DELAY	MACRO	CYCLES
	IF	((CYCLES>3)&&(CYCLES<8))
	CALL	DEL4
	ENDIF
	IF	(CYCLES>7)
	MOVLW	((CYCLES/4)-1)
	CALL	DEL4N
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>0)
	NOP
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>1)
	NOP
	ENDIF
	IF	((CYCLES-(4*(CYCLES/4)))>2)
	NOP
	ENDIF
	ENDM

; Unpack text line macro
; Expects STATUS image in W
; Expects IRP:FSR to point at PORTC
; Expects char 21 in TMP2
; Unpacks as follows:
;  char 22 to TMP3
;  char 23 to TMP4
;  char 24 to TMP5
; Generates sync pulse at +1~
UNPACK	MACRO	BASE			; UNPACK = 45~
	MOVWF	STATUS
	BCF	INDF,SYNC
	RLF	BASE,W
	RLF	TMP3
	RLF	BASE+1,W
	RLF	TMP3
	RLF	BASE+2,W
	RLF	TMP3
	RLF	BASE+3,W
	RLF	TMP3
	RLF	BASE+4,W
	RLF	TMP3
	RLF	BASE+5,W
	RLF	TMP3
	RLF	BASE+6,W
	RLF	TMP3
	RLF	BASE+7,W
	RLF	TMP4
	RLF	BASE+8,W
	RLF	TMP4
	RLF	BASE+9,W
	RLF	TMP4
	RLF	BASE+D'10',W
	RLF	TMP4
	RLF	BASE+D'11',W
	RLF	TMP4
	BSF	INDF,SYNC
	RLF	BASE+D'12',W
	RLF	TMP4
	RLF	BASE+D'13',W
	RLF	TMP4
	RLF	BASE+D'14',W
	RLF	TMP5
	RLF	BASE+D'15',W
	RLF	TMP5
	RLF	BASE+D'16',W
	RLF	TMP5
	RLF	BASE+D'17',W
	RLF	TMP5
	RLF	BASE+D'18',W
	RLF	TMP5
	RLF	BASE+D'19',W
	RLF	TMP5
	RLF	TMP2,W
	RLF	TMP5
	ENDM

; Display pixel line macro
; Expects STATUS image in W
; Expects IRP:FSR to point at TXREG
; Expects text line unpacked as follows:
;  char 21 in TMP2
;  char 22 in TMP3
;  char 23 in TMP4
;  char 24 in TMP5
;  char 25 in TMP6
; Expects character ROM page in TMP7
; Zero sent to TXREG at +3~
; First character sent to TXREG at +11~
DSPLY	MACRO	BASE			; DSPLY = 205~
	MOVWF	STATUS
	MOVF	TMP7,W
	MOVWF	PCLATH
	CLRF	INDF
	MOVF	BASE,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+1,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+2,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+3,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+4,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+5,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+6,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+7,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+8,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+9,W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'10',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'11',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'12',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'13',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'14',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'15',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'16',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'17',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'18',W
	CALL	0
	MOVWF	INDF
	MOVF	BASE+D'19',W
	CALL	0
	MOVWF	INDF
	MOVF	TMP2,W
	CALL	0
	MOVWF	INDF
	MOVF	TMP3,W
	CALL	0
	MOVWF	INDF
	MOVF	TMP4,W
	CALL	0
	MOVWF	INDF
	MOVF	TMP5,W
	CALL	0
	MOVWF	INDF
	MOVF	TMP6,W
	CALL	0
	MOVWF	INDF
	CLRF	PCLATH
	ENDM

; Clear a text line macro
; Expects STATUS image in W
; Does not clear char 21, 25
CLRTXT	MACRO	BASE				; CLRTXT = 23~
	MOVWF	STATUS
	MOVLW	SPCH
	MOVWF	BASE
	MOVWF	BASE+2
	MOVWF	BASE+3
	MOVWF	BASE+4
	MOVWF	BASE+5
	MOVWF	BASE+6
	MOVWF	BASE+7
	MOVWF	BASE+9
	MOVWF	BASE+D'10'
	MOVWF	BASE+D'11'
	MOVWF	BASE+D'12'
	MOVWF	BASE+D'13'
	MOVWF	BASE+D'14'
	MOVWF	BASE+D'16'
	MOVWF	BASE+D'17'
	MOVWF	BASE+D'18'
	MOVWF	BASE+D'19'
	MOVLW	SPCH7
	MOVWF	BASE+1
	MOVWF	BASE+8
	MOVWF	BASE+D'15'
	ENDM

; Table check macro
TBLCHK	MACRO	FUNC
	IF	HIGH($)!=HIGH(FUNC)
	ERROR	"TABLE CROSSING PAGE BOUNDARY"
	ENDIF
	ENDM

; Reset vector
	ORG	0
	GOTO	RESET

; Main loop
L1	BCF	PORTC,SYNC		; Frame sync (6 broad pulses)
	CALL	POLLCO
	CALL	POLCI1
	CALL	POLCI2
	DELAY	D'59'
	BSF	PORTC,SYNC
	DELAY	D'24'
	BCF	PORTC,SYNC
	DELAY	D'134'
	BSF	PORTC,SYNC
	DELAY	D'24'
	BCF	PORTC,SYNC
	CALL	POLLCO
	CALL	POLCI1
	CALL	POLCI2
	DELAY	D'59'
	BSF	PORTC,SYNC
	DELAY	D'24'
	BCF	PORTC,SYNC
	DELAY	D'134'
	BSF	PORTC,SYNC
	DELAY	D'24'
	BCF	PORTC,SYNC
	CALL	POLLCO
	CALL	POLCI1
	CALL	POLCI2
	DELAY	D'59'
	BSF	PORTC,SYNC
	DELAY	D'24'
	BCF	PORTC,SYNC
	DELAY	D'129'
	MOVLW	D'51'			; Pre-blanking scan lines
	MOVWF	TMP1			;  (PAL:51 NTSC:24)
L2	DELAY	2
	MOVLW	B'01100000'
	MOVWF	PORTC
	DELAY	D'24'
	BCF	PORTC,SYNC
	CALL	POLLCO
	BSF	PORTC,SYNC
	CALL	PREFNC
	DECFSZ	TMP1
	GOTO	L2
	MOVLW	D'16'			; 16 text lines to display
	MOVWF	TMP1
L3	DELAY	6
	MOVF	VIDSTT,W		; Unpack next text line
	ANDLW	H'0F'
	ADDLW	H'90'
	MOVWF	FSR
	BSF	STATUS,IRP
	MOVF	INDF,W
	MOVWF	TMP2
	BCF	FSR,7
	MOVF	INDF,W
	MOVWF	TMP6
	MOVLW	PORTC
	MOVWF	FSR
	BCF	STATUS,IRP
	RLF	VIDSTT,W
	ANDLW	H'1E'
	ADDWF	PCL
	MOVLW	STATP0
	GOTO	UPCB32
	MOVLW	STATP0
	GOTO	UPCB52
	MOVLW	STATP0
	GOTO	UPCB72
	MOVLW	STATP0
	GOTO	UPCB92
	MOVLW	STATP1
	GOTO	UPCB32
	MOVLW	STATP1
	GOTO	UPCB52
	MOVLW	STATP1
	GOTO	UPCB72
	MOVLW	STATP1
	GOTO	UPCB92
	MOVLW	STATP2
	GOTO	UPCB32
	MOVLW	STATP2
	GOTO	UPCB52
	MOVLW	STATP2
	GOTO	UPCB72
	MOVLW	STATP2
	GOTO	UPCB92
	MOVLW	STATP3
	GOTO	UPCB32
	MOVLW	STATP3
	GOTO	UPCB52
	MOVLW	STATP3
	GOTO	UPCB72
	MOVLW	STATP3
	GOTO	UPCB92
L4	CLRF	STATUS
	BSF	RCSTA,SPEN
	DELAY	1
	CLRF	TXREG
	MOVLW	D'18'			; 14 scan lines per text line
	MOVWF	TMP7			; (character ROM pages 18..31)
	BCF	STATUS,IRP
	CLRF	TXREG
	GOTO	L6
L5	CALL	POLCI1
	BCF	PORTC,SYNC
	CALL	POLLCO
	BSF	PORTC,SYNC
	CALL	POLCI2
	BSF	RCSTA,SPEN
	DELAY	1
	CLRF	TXREG
	DELAY	2
	BCF	STATUS,IRP
	CLRF	TXREG
	DELAY	2
L6	MOVLW	TXREG			; Display a row of pixels
	MOVWF	FSR
	RLF	VIDSTT,W
	ANDLW	H'1E'
	CLRF	TXREG
	ADDWF	PCL
	MOVLW	STATP0
	GOTO	DSPB32
	MOVLW	STATP0
	GOTO	DSPB52
	MOVLW	STATP0
	GOTO	DSPB72
	MOVLW	STATP0
	GOTO	DSPB92
	MOVLW	STATP1
	GOTO	DSPB32
	MOVLW	STATP1
	GOTO	DSPB52
	MOVLW	STATP1
	GOTO	DSPB72
	MOVLW	STATP1
	GOTO	DSPB92
	MOVLW	STATP2
	GOTO	DSPB32
	MOVLW	STATP2
	GOTO	DSPB52
	MOVLW	STATP2
	GOTO	DSPB72
	MOVLW	STATP2
	GOTO	DSPB92
	MOVLW	STATP3
	GOTO	DSPB32
	MOVLW	STATP3
	GOTO	DSPB52
	MOVLW	STATP3
	GOTO	DSPB72
	MOVLW	STATP3
	GOTO	DSPB92
	TBLCHK	L1
L7	CLRF	STATUS
	DELAY	3
	CLRF	TXREG
	DELAY	7
	BCF	RCSTA,SPEN
	INCF	TMP7
	BTFSS	TMP7,5
	GOTO	L5
	INCF	VIDSTT
	DECFSZ	TMP1
	GOTO	L3
	MOVLW	D'34'			; Post-blanking scan lines
	MOVWF	TMP1			;  (PAL:34 NTSC:11)
L8	CALL	POLLCO
	DELAY	1
	MOVLW	B'01000000'
	MOVWF	PORTC
	DELAY	D'23'
	MOVLW	B'01100000'
	MOVWF	PORTC
	CALL	PSTFNC
	DECFSZ	TMP1
	GOTO	L8
	DELAY	D'22'
	BTFSC	FLAGS,BELLF
	BCF	PORTC,BLEEP
	BCF	FLAGS,BELLF
	GOTO	L1

; Reset
RESET	MOVLW	B'00100000'
	MOVWF	PORTA
	MOVLW	B'01100000'
	MOVWF	PORTC
	BSF	STATUS,RP0
	MOVLW	B'00010111'
	MOVWF	TRISE
	MOVLW	6
	MOVWF	ADCON1
	MOVLW	B'00011011'
	MOVWF	TRISC
	MOVLW	D'173'
	MOVWF	PR2
	MOVLW	B'10110000'
	MOVWF	TXSTA
	CLRF	SPBRG
	MOVLW	B'00000100'
	MOVWF	OPTREG
	MOVLW	B'00011111'
	MOVWF	TRISA
	BCF	STATUS,RP0
	MOVLW	D'87'
	MOVWF	CCPR1L
	MOVLW	B'00000010'
	MOVWF	T2CON
	MOVLW	B'00000101'
	MOVWF	CCP2CN
	MOVLW	B'00001100'
	MOVWF	CCP1CN
	BSF	T1CON,0
	BSF	T2CON,2
	CLRF	PORTD
	MOVF	PORTD,W

; Program initialisation
	CLRF	TMP1
L9	MOVF	TMP1,W
	CALL	CLRLIN
	INCF	TMP1
	BTFSS	TMP1,4
	GOTO	L9
	CLRF	VIDSTT
	CLRF	CURV
	CLRF	CURH
	MOVLW	H'80'
	MOVWF	KEY1
	MOVWF	MOD1
	MOVWF	KEY2
	MOVWF	MOD2
	CLRF	COCNT
	MOVWF	COBUF1
	MOVWF	COBUF2
	CLRF	FLAGS
	BSF	STATUS,RP0
	MOVLW	CDTO
	MOVWF	CDCNT
	MOVLW	1
	MOVWF	SEQCNT
	MOVLW	H'FF'
	MOVWF	CISR
	BCF	STATUS,RP0
	BSF	STATUS,RP1
	MOVWF	CIBUF1
	MOVWF	CIBUF2
	BCF	STATUS,RP1
	MOVLW	CURCH
	CALL	WRSMEM
	GOTO	L1

; Clear a line
; Line number 0..15 expected in W
; Uses TMP5
CLRLIN	MOVWF	TMP5			; CALL CLRLIN = 49~
	MOVLW	HIGH(CLRLIN)
	MOVWF	PCLATH
	RLF	TMP5,W
	ANDLW	H'1E'
	ADDWF	PCL
	MOVLW	STATP0
	GOTO	CLRB32
	MOVLW	STATP0
	GOTO	CLRB52
	MOVLW	STATP0
	GOTO	CLRB72
	MOVLW	STATP0
	GOTO	CLRB92
	MOVLW	STATP1
	GOTO	CLRB32
	MOVLW	STATP1
	GOTO	CLRB52
	MOVLW	STATP1
	GOTO	CLRB72
	MOVLW	STATP1
	GOTO	CLRB92
	MOVLW	STATP2
	GOTO	CLRB32
	MOVLW	STATP2
	GOTO	CLRB52
	MOVLW	STATP2
	GOTO	CLRB72
	MOVLW	STATP2
	GOTO	CLRB92
	MOVLW	STATP3
	GOTO	CLRB32
	MOVLW	STATP3
	GOTO	CLRB52
	MOVLW	STATP3
	GOTO	CLRB72
	MOVLW	STATP3
	GOTO	CLRB92
	TBLCHK	CLRLIN
L10	CLRF	STATUS
	CLRF	PCLATH
	MOVF	TMP5,W
	ADDLW	H'90'
	MOVWF	FSR
	BSF	STATUS,IRP
	MOVLW	SPCH
	MOVWF	INDF
	BCF	FSR,7
	MOVWF	INDF
	RETURN

; Write character to screen memory
; Character expected in W
; Null characters converted to spaces
; Cursor vertical position 0..15 expected in CURV
; Cursor horizontal position 0..24 expected in CURH
; Uses TMP2, TMP3
WRSMEM	MOVWF	TMP2			; CALL WRSMEM = 63~
	BCF	TMP2,7
	MOVLW	SPCH
	MOVF	TMP2
	BTFSC	STATUS,Z
	MOVWF	TMP2
	MOVLW	HIGH(WRSMEM)
	MOVWF	PCLATH
	RLF	CURV,W
	MOVWF	TMP3
	RLF	TMP3,W
	ANDLW	H'3C'
	ADDWF	PCL
	BCF	STATUS,IRP
	MOVLW	D'32'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'52'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'72'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'92'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'160'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'180'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'200'
	GOTO	L11
	NOP
	BCF	STATUS,IRP
	MOVLW	D'220'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'32'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'52'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'72'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'92'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'160'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'180'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'200'
	GOTO	L11
	NOP
	BSF	STATUS,IRP
	MOVLW	D'220'
	GOTO	L11
L11	MOVWF	FSR
	RLF	CURH,W
	ANDLW	H'3E'
	ADDWF	PCL
	MOVLW	0
	GOTO	L12
	MOVLW	1
	GOTO	L12
	MOVLW	2
	GOTO	L12
	MOVLW	3
	GOTO	L12
	MOVLW	4
	GOTO	L12
	MOVLW	5
	GOTO	L12
	MOVLW	6
	GOTO	L12
	MOVLW	7
	GOTO	L12
	MOVLW	8
	GOTO	L12
	MOVLW	9
	GOTO	L12
	MOVLW	D'10'
	GOTO	L12
	MOVLW	D'11'
	GOTO	L12
	MOVLW	D'12'
	GOTO	L12
	MOVLW	D'13'
	GOTO	L12
	MOVLW	D'14'
	GOTO	L12
	MOVLW	D'15'
	GOTO	L12
	MOVLW	D'16'
	GOTO	L12
	MOVLW	D'17'
	GOTO	L12
	MOVLW	D'18'
	GOTO	L12
	MOVLW	D'19'
	GOTO	L12
	NOP
	GOTO	L13
	MOVLW	0
	GOTO	L14
	MOVLW	7
	GOTO	L14
	MOVLW	D'14'
	GOTO	L15
	NOP
	GOTO	L16
	TBLCHK	WRSMEM
L12	ADDWF	FSR
	CLRF	PCLATH
	RLF	TMP2
	RLF	INDF
	RRF	TMP2,W
	MOVWF	INDF
	DELAY	D'27'
	RETURN
L13	CLRF	PCLATH
	MOVF	CURV,W
	ADDLW	H'90'
	MOVWF	FSR
	BSF	STATUS,IRP
	RLF	TMP2
	RLF	INDF
	RRF	TMP2,W
	MOVWF	INDF
	DELAY	D'24'
	RETURN
L14	ADDWF	FSR
	CLRF	PCLATH
	RLF	TMP2
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	DELAY	3
	RETURN
L15	ADDWF	FSR
	CLRF	PCLATH
	RLF	TMP2
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	INCF	FSR
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	MOVF	CURV,W
	ADDLW	H'90'
	MOVWF	FSR
	BSF	STATUS,IRP
	RLF	INDF
	RLF	TMP2
	RRF	INDF
	RETURN
L16	CLRF	PCLATH
	MOVF	CURV,W
	ADDLW	H'10'
	MOVWF	FSR
	BSF	STATUS,IRP
	MOVF	TMP2,W
	MOVWF	INDF
	DELAY	D'26'
	RETURN

; Pre-blanking functions
; Expects index 51..1 in TMP1
PREFNC	MOVLW	HIGH(PREFNC)		; CALL PREFNC = 263~
	MOVWF	PCLATH
	MOVF	TMP1,W
	ADDWF	PCL
	NOP
	GOTO	PRECO
	GOTO	PRENUL
	GOTO	PRECI
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRECO
	GOTO	PRENUL
	GOTO	PRECI
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRENUL
	GOTO	PRECO
	GOTO	PRENUL
	GOTO	PRECI
	TBLCHK	PREFNC
PRENUL	CLRF	PCLATH
	CALL	POLCI1
	CALL	POLCI2
	DELAY	D'200'
	RETURN

; General pre blanking functions
PRECO	CLRF	PCLATH
	DELAY	D'37'
	GOTO	DSPCO
PRECI	CLRF	PCLATH
	DELAY	D'35'
	GOTO	DSPCI

; Post-blanking functions
; Expects index 34..1 in TMP1
PSTFNC	MOVLW	HIGH(PSTFNC)		; CALL PSTFNC = 265~
	MOVWF	PCLATH
	MOVF	TMP1,W
	ADDWF	PCL
	NOP
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	PSTNUL
	GOTO	KDEC2
	GOTO	PSTNUL
	GOTO	KDEC1
	GOTO	PSTNUL
	GOTO	KBSCNA
	GOTO	PSTNUL
	GOTO	KBSCN9
	GOTO	PSTNUL
	GOTO	KBSCN8
	GOTO	PSTNUL
	GOTO	KBSCN7
	GOTO	PSTNUL
	GOTO	KBSCN6
	GOTO	PSTNUL
	GOTO	KBSCN5
	GOTO	PSTNUL
	GOTO	KBSCN4
	GOTO	PSTNUL
	GOTO	KBSCN3
	GOTO	PSTNUL
	GOTO	KBSCN2
	GOTO	PSTNUL
	GOTO	KBSCN1
	GOTO	PSTNUL
	GOTO	PSTCO
	GOTO	PSTNUL
	GOTO	PSTCI
	TBLCHK	PSTFNC
PSTNUL	CLRF	PCLATH
	CALL	POLCI1
	CALL	POLCI2
	DELAY	D'202'
	RETURN

; Keyboard scanning post blanking functions
; Called 128 microseconds apart
; Use TMP2
KBSCN1	CLRF	PCLATH
	BCF	FLAGS,SHFTF
	BCF	FLAGS,CTRLF
	BSF	STATUS,RP0
	BCF	TRISA,COL1
	BCF	STATUS,RP0
	DELAY	D'248'
	RETURN
KBSCN2	CLRF	PCLATH
	BTFSC	PORTA,ROW9
	BSF	FLAGS,SHFTF
	BSF	STATUS,RP0
	BSF	TRISA,COL1
	BCF	TRISA,COL2
	BCF	STATUS,RP0
	DELAY	D'247'
	RETURN
KBSCN3	CLRF	PCLATH
	BTFSC	PORTA,ROW9
	BSF	FLAGS,CTRLF
	BSF	STATUS,RP0
	BSF	TRISA,COL2
	BCF	TRISA,COL1
	BCF	STATUS,RP0
	DELAY	D'247'
	RETURN
KBSCN4	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISA,COL1
	BCF	TRISA,COL2
	BCF	STATUS,RP0
	CLRF	TMP2
	GOTO	KBMSCN
KBSCN5	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISA,COL2
	BCF	TRISA,COL3
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN
KBSCN6	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISA,COL3
	BCF	TRISA,COL4
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN
KBSCN7	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISA,COL4
	BCF	TRISC,COL5
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN
KBSCN8	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISC,COL5
	BCF	TRISC,COL6
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN
KBSCN9	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISC,COL6
	BCF	TRISC,COL7
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN
KBSCNA	MOVF	PORTB,W
	BSF	STATUS,RP0
	BSF	TRISC,COL7
	DELAY	1
	BCF	STATUS,RP0
	INCF	TMP2
	GOTO	KBMSCN

; Cassette display post blanking functions
PSTCO	CLRF	PCLATH
	DELAY	D'39'
	GOTO	DSPCO
PSTCI	CLRF	PCLATH
	DELAY	D'37'
	GOTO	DSPCI

; Keyboard minor scan
; Expects column image in W
; Expects column index 0..6 in TMP2
; Uses TMP3
; KEYn bit 7 = no key flag
; MODn bit 7 = no new key flag
KBMSCN	CLRF	PCLATH			; CALL KBMSCN = 250~
	MOVWF	TMP3
	MOVLW	7
	ANDWF	TMP2
L17	BTFSC	TMP3,0
	GOTO	L18
	MOVF	TMP2,W
	XORWF	KEY1,W
	BTFSC	STATUS,Z
	GOTO	L19
	MOVF	TMP2,W
	XORWF	KEY2,W
	BTFSC	STATUS,Z
	GOTO	L20
	BTFSS	KEY1,7
	GOTO	L22
	MOVF	TMP2,W
	MOVWF	KEY1
	MOVF	FLAGS,W
	MOVWF	MOD1
	DELAY	1
	GOTO	L23
L18	MOVF	TMP2,W
	XORWF	KEY1,W
	BTFSC	STATUS,Z
	BSF	KEY1,7
	BTFSC	STATUS,Z
	BSF	MOD1,7
	MOVF	TMP2,W
	XORWF	KEY2,W
	BTFSC	STATUS,Z
	BSF	KEY2,7
	BTFSC	STATUS,Z
	BSF	MOD2,7
	DELAY	2
	GOTO	L23
L19	DELAY	4
L20	DELAY	5
L21	DELAY	1
	GOTO	L23
L22	BTFSS	KEY2,7
	GOTO	L21
	MOVF	TMP2,W
	MOVWF	KEY2
	MOVF	FLAGS,W
	MOVWF	MOD2
L23	RRF	TMP3
	MOVLW	8
	ADDWF	TMP2
	BTFSS	TMP2,6
	GOTO	L17
	DELAY	D'43'
	RETURN

; Deliver character to display
; Character expected in W
; Uses TMP4
SNDDSP	ADDLW	H'E0'			; CALL SNDDSP = 205~
	BTFSS	STATUS,C
	GOTO	L25
	ADDLW	H'20'
	BTFSC	FLAGS,ESCF
	ANDLW	H'1F'
	GOTO	PRCHR
L25	ADDLW	H'20'
	MOVWF	TMP4
	MOVLW	HIGH(SNDDSP)
	MOVWF	PCLATH
	MOVF	TMP4,W
	ADDWF	PCL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHBEL
	GOTO	CCHBS
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHCR
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHESC
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	GOTO	CCHNUL
	TBLCHK	SNDDSP
CCHNUL	CLRF	PCLATH
	DELAY	D'187'
	RETURN
CCHCR	CLRF	PCLATH
	MOVLW	SPCH
	CALL	WRSMEM
	CLRF	CURH
	INCF	CURV
	BCF	CURV,4
	MOVF	CURV,W
	XORWF	VIDSTT,W
	ANDLW	H'0F'
	BTFSS	STATUS,Z
	GOTO	L26
	INCF	VIDSTT
	MOVF	CURV,W
	CALL	CLRLIN
	MOVLW	CURCH
	CALL	WRSMEM
	RETURN
L26	MOVLW	CURCH
	CALL	WRSMEM
	DELAY	D'50'
	RETURN
CCHBS	CLRF	PCLATH
	MOVLW	SPCH
	CALL	WRSMEM
	DECF	CURH
	BTFSC	CURH,7
	DECF	CURV
	MOVLW	D'24'
	BTFSC	CURH,7
	MOVWF	CURH
	MOVLW	H'0F'
	ANDWF	CURV
	MOVLW	CURCH
	CALL	WRSMEM
	DELAY	D'51'
	RETURN
CCHESC	CLRF	PCLATH
	BSF	FLAGS,ESCF
	DELAY	D'186'
	RETURN
CCHBEL	CLRF	PCLATH
	BSF	FLAGS,BELLF
	DELAY	D'186'
	RETURN
PRCHR	CALL	WRSMEM
	INCF	CURH
	MOVF	CURH,W
	ADDLW	D'231'
	BTFSS	STATUS,C
	GOTO	L27
	CLRF	CURH
	INCF	CURV
	BCF	CURV,4
	MOVF	CURV,W
	XORWF	VIDSTT,W
	ANDLW	H'0F'
	BTFSS	STATUS,Z
	GOTO	L28
	INCF	VIDSTT
	MOVF	CURV,W
	CALL	CLRLIN
	BCF	FLAGS,ESCF
	DELAY	1
	MOVLW	CURCH
	CALL	WRSMEM
	RETURN
L27	DELAY	8
L28	DELAY	D'51'
	BCF	FLAGS,ESCF
	MOVLW	CURCH
	CALL	WRSMEM
	RETURN

; Poll cassette output
POLLCO	MOVF	COCNT			; CALL POLLCO = 24~
	BTFSC	STATUS,Z
	GOTO	L29
	MOVLW	B'00100000'
	MOVWF	PORTA
	BTFSS	PIR1,TMR2IF
	GOTO	L30
	MOVLW	D'174'
	BTFSC	COCNT,0
	MOVLW	0
	BTFSC	COSR,0
	MOVLW	D'87'
	MOVWF	CCPR1L
	BCF	PIR1,TMR2IF
	BTFSC	COCNT,0
	RRF	COSR
	BTFSC	COCNT,0
	BSF	COSR,7
	DECF	COCNT
	DELAY	1
	RETURN
L29	BSF	STATUS,RP0
	BTFSS	TRISE,IBF
	GOTO	L30
	BCF	STATUS,RP0
	MOVF	COBUF1,W
	MOVWF	COBUF2
	MOVF	PORTD,W
	MOVWF	COBUF1
	RLF	COBUF1,W
	RLF	COBUF1,W
	MOVWF	COSR
	BCF	PIR1,TMR2IF
	MOVLW	D'18'
	MOVWF	COCNT
	CLRF	PORTA
	DELAY	1
	RETURN
L30	DELAY	D'11'
	BCF	STATUS,RP0
	RETURN

; Poll cassette input (part 1)
POLCI1	BTFSC	INTCON,T0IF		; CALL POLCI1 = 30~
	GOTO	L32
	BTFSS	PIR2,CCP2IF
	GOTO	L33
	CLRF	TMR0
	BCF	INTCON,T0IF
	MOVF	CCPR2H,W
	BCF	PIR2,CCP2IF
	BSF	STATUS,RP0
	ADDWF	PRVCAP,W
	SUBWF	PRVCAP
	DECFSZ	CDCNT
	GOTO	L34
	INCF	CDCNT
	SUBLW	D'16'
	INCF	CIDIV
	BTFSS	STATUS,C
	CLRF	CIDIV
	BTFSS	CIDIV,0
	RRF	CISR
	BTFSC	CISR,0
	GOTO	L35
	RRF	SEQCNT,W
	RRF	CISR,W
	BCF	STATUS,RP0
	MOVWF	PORTD
	RETURN
L32	BCF	INTCON,T0IF
	BSF	STATUS,RP0
	MOVLW	CDTO
	MOVWF	CDCNT
	MOVLW	H'FF'
	MOVWF	CISR
	BCF	STATUS,RP0
	DELAY	D'16'
	RETURN
L33	DELAY	9
L34	DELAY	9
L35	DELAY	2
	BCF	STATUS,RP0
	RETURN

; Poll cassette input (part 2)
POLCI2	BSF	STATUS,RP0		; CALL POLCI2 = 21~
	BTFSC	CISR,0
	GOTO	L36
	COMF	CISR,W
	IORWF	CISR
	XORLW	H'FF'
	INCF	SEQCNT
	BCF	STATUS,RP0
	BSF	STATUS,RP1
	XORWF	CIBUF1,W
	MOVWF	CIBUF2
	XORWF	CIBUF1,W
	MOVWF	CIBUF1
	XORWF	CIBUF2
	RRF	CIBUF1
	BCF	CIBUF1,7
	BCF	STATUS,RP1
	RETURN
L36	BCF	STATUS,RP0
	DELAY	D'12'
	RETURN

; Display one character of cassette input
DSPCI	BSF	STATUS,RP1		; CALL DSPCI = 218~
	BTFSC	CIBUF2,7
	GOTO	L37
	DELAY	1
	MOVF	CIBUF2,W
	BSF	CIBUF2,7
	GOTO	L38
L37	BTFSC	CIBUF1,7
	GOTO	L39
	MOVF	CIBUF1,W
	BSF	CIBUF1,7
L38	BCF	STATUS,RP1
	CALL	SNDDSP
	RETURN
L39	BCF	STATUS,RP1
	DELAY	D'206'
	RETURN

; Display one character of cassette output
DSPCO	BTFSC	COBUF2,7		; CALL DSPCO = 216~
	GOTO	L40
	DELAY	1
	MOVF	COBUF2,W
	BSF	COBUF2,7
	GOTO	L41
L40	BTFSC	COBUF1,7
	GOTO	L42
	MOVF	COBUF1,W
	BSF	COBUF1,7
L41	CALL	SNDDSP
	RETURN
L42	DELAY	D'206'
	RETURN

; Key decode post blanking functions
; Use TMP6
KDEC1	CLRF	PCLATH
	BTFSC	MOD1,7
	GOTO	L47
	CALL	L48
	CLRF	PCLATH
	BSF	MOD1,7
	MOVWF	TMP6
	MOVF	COCNT
	BTFSS	STATUS,Z
	GOTO	L43
	MOVWF	COSR
	RLF	COSR
	BCF	COSR,0
	BCF	PIR1,TMR2IF
	MOVLW	D'18'
	MOVWF	COCNT
	GOTO	L44
L43	DELAY	7
L44	MOVF	TMP6,W
	CALL	SNDDSP
	RLF	TMP6
	BSF	STATUS,RP0
	RRF	SEQCNT,W
	INCF	SEQCNT
	BCF	STATUS,RP0
	RRF	TMP6,W
	MOVWF	PORTD
	DELAY	D'11'
	RETURN
KDEC2	CLRF	PCLATH
	BTFSC	MOD2,7
	GOTO	L47
	CALL	L49
	CLRF	PCLATH
	BSF	MOD2,7
	MOVWF	TMP6
	MOVF	COCNT
	BTFSS	STATUS,Z
	GOTO	L45
	MOVWF	COSR
	RLF	COSR
	BCF	COSR,0
	BCF	PIR1,TMR2IF
	MOVLW	D'18'
	MOVWF	COCNT
	GOTO	L46
L45	DELAY	7
L46	MOVF	TMP6,W
	CALL	SNDDSP
	RLF	TMP6
	BSF	STATUS,RP0
	RRF	SEQCNT,W
	INCF	SEQCNT
	BCF	STATUS,RP0
	RRF	TMP6,W
	MOVWF	PORTD
	DELAY	D'11'
	RETURN
L47	DELAY	D'250'
	RETURN
L48	MOVLW	HIGH(KSCTAB)
	MOVWF	PCLATH
	MOVF	KEY1,W
	BTFSS	MOD1,SHFTF
	IORLW	H'40'
	BTFSS	MOD1,CTRLF
	IORLW	H'80'
	MOVWF	PCL
L49	MOVLW	HIGH(KSCTAB)
	MOVWF	PCLATH
	MOVF	KEY2,W
	BTFSS	MOD2,SHFTF
	IORLW	H'40'
	BTFSS	MOD2,CTRLF
	IORLW	H'80'
	MOVWF	PCL

; Delay routines
DEL4N	ADDLW	H'FF'			; CALL DEL4N = ((4*(W+1))-1)~
	BTFSS	STATUS,Z
	GOTO	DEL4N
DEL4	RETURN				; CALL DEL4 = 4~

; Unpack text line specific functions
UPCB32	UNPACK	D'32'
	GOTO	L4
UPCB52	UNPACK	D'52'
	GOTO	L4
UPCB72	UNPACK	D'72'
	GOTO	L4
UPCB92	UNPACK	D'92'
	GOTO	L4

; Display text line specific functions
DSPB32	DSPLY	D'32'
	GOTO	L7
DSPB52	DSPLY	D'52'
	GOTO	L7
DSPB72	DSPLY	D'72'
	GOTO	L7
DSPB92	DSPLY	D'92'
	GOTO	L7

; Clear text line specific functions
CLRB32	CLRTXT	D'32'
	GOTO	L10
CLRB52	CLRTXT	D'52'
	GOTO	L10
CLRB72	CLRTXT	D'72'
	GOTO	L10
CLRB92	CLRTXT	D'92'
	GOTO	L10

; This is needed by character ROM look-up code
	ORG	H'1000'
	MOVWF	PCL

; Keyboard scan code table
	ORG	H'1100'
KSCTAB
	IF	UPPERC
	DT	ESCCH,	'1',	'2',	'3',	'4',	'5',	'6',	0	; Neither
	DT	TABCH,	'Q',	'W',	'E',	'R',	'T',	'Y',	0
	DT	0,	'A',	'S',	'D',	'F',	'G',	'H',	0
	DT	BSLCH,	'Z',	'X',	'C',	'V',	'B',	'N',	0
	DT	'7',	'8',	'9',	'0',	'-',	'=',	BSCH,	0
	DT	'U',	'I',	'O',	'P',	'[',	']',	RETCH,	0
	DT	'J',	'K',	'L',	';',	APOCH,	'#',	0,	0
	DT	'M',	',',	'.',	'/',	SPCH,	0,	'N',	0

	DT	ESCCH,	'!',	'"',	PNDCH,	'$',	'%',	'^',	0	; Shift
	DT	BTABCH,	'q',	'w',	'e',	'r',	't',	'y',	0
	DT	0,	'a',	's',	'd',	'f',	'g',	'h',	0
	DT	'|',	'z',	'x',	'c',	'v',	'b',	'n',	0
	DT	'&',	'*',	'(',	')',	'_',	'+',	BSCH,	0
	DT	'u',	'i',	'o',	'p',	'{',	'}',	RETCH,	0
	DT	'j',	'k',	'l',	':',	'@',	'~',	0,	0
	DT	'm',	'<',	'>',	'?',	SPCH,	0,	'n',	0

	DT	ESCCH,	'1',	'2',	'3',	'4',	'5',	'6',	0	; Control
	DT	TABCH,	H'11',	H'17',	H'05',	H'12',	H'14',	H'19',	0
	DT	0,	H'01',	H'13',	H'04',	H'06',	H'07',	H'08',	0
	DT	H'1C',	H'1A',	H'18',	H'03',	H'16',	H'02',	H'0E',	0
	DT	'7',	'8',	'9',	'0',	'-',	'=',	BSCH,	0
	DT	H'15',	H'09',	H'0F',	H'10',	H'1B',	H'1D',	RETCH,	0
	DT	H'0A',	H'0B',	H'0C',	';',	APOCH,	'#',	0,	0
	DT	H'0D',	',',	'.',	'/',	SPCH,	0,	H'0E',	0

	DT	ESCCH,	'!',	'"',	PNDCH,	'$',	'%',	H'1E',	0	; Both
	DT	BTABCH,	H'11',	H'17',	H'05',	H'12',	H'14',	H'19',	0
	DT	0,	H'01',	H'13',	H'04',	H'06',	H'07',	H'08',	0
	DT	H'1C',	H'1A',	H'18',	H'03',	H'16',	H'02',	H'0E',	0
	DT	'&',	'*',	'(',	')',	H'1F',	'+',	BSCH,	0
	DT	H'15',	H'09',	H'0F',	H'10',	H'1B',	H'1D',	RETCH,	0
	DT	H'0A',	H'0B',	H'0C',	':',	'@',	'~',	0,	0
	DT	H'0D',	'<',	'>',	'?',	SPCH,	0,	H'0E',	0
	ELSE
	DT	ESCCH,	'1',	'2',	'3',	'4',	'5',	'6',	0	; Neither
	DT	TABCH,	'q',	'w',	'e',	'r',	't',	'y',	0
	DT	0,	'a',	's',	'd',	'f',	'g',	'h',	0
	DT	BSLCH,	'z',	'x',	'c',	'v',	'b',	'n',	0
	DT	'7',	'8',	'9',	'0',	'-',	'=',	BSCH,	0
	DT	'u',	'i',	'o',	'p',	'[',	']',	RETCH,	0
	DT	'j',	'k',	'l',	';',	APOCH,	'#',	0,	0
	DT	'm',	',',	'.',	'/',	SPCH,	0,	'n',	0

	DT	ESCCH,	'!',	'"',	PNDCH,	'$',	'%',	'^',	0	; Shift
	DT	BTABCH,	'Q',	'W',	'E',	'R',	'T',	'Y',	0
	DT	0,	'A',	'S',	'D',	'F',	'G',	'H',	0
	DT	'|',	'Z',	'X',	'C',	'V',	'B',	'N',	0
	DT	'&',	'*',	'(',	')',	'_',	'+',	BSCH,	0
	DT	'U',	'I',	'O',	'P',	'{',	'}',	RETCH,	0
	DT	'J',	'K',	'L',	':',	'@',	'~',	0,	0
	DT	'M',	'<',	'>',	'?',	SPCH,	0,	'N',	0

	DT	ESCCH,	'1',	'2',	'3',	'4',	'5',	'6',	0	; Control
	DT	TABCH,	H'11',	H'17',	H'05',	H'12',	H'14',	H'19',	0
	DT	0,	H'01',	H'13',	H'04',	H'06',	H'07',	H'08',	0
	DT	H'1C',	H'1A',	H'18',	H'03',	H'16',	H'02',	H'0E',	0
	DT	'7',	'8',	'9',	'0',	'-',	'=',	BSCH,	0
	DT	H'15',	H'09',	H'0F',	H'10',	H'1B',	H'1D',	RETCH,	0
	DT	H'0A',	H'0B',	H'0C',	';',	APOCH,	'#',	0,	0
	DT	H'0D',	',',	'.',	'/',	SPCH,	0,	H'0E',	0

	DT	ESCCH,	'!',	'"',	PNDCH,	'$',	'%',	H'1E',	0	; Both
	DT	BTABCH,	H'11',	H'17',	H'05',	H'12',	H'14',	H'19',	0
	DT	0,	H'01',	H'13',	H'04',	H'06',	H'07',	H'08',	0
	DT	H'1C',	H'1A',	H'18',	H'03',	H'16',	H'02',	H'0E',	0
	DT	'&',	'*',	'(',	')',	H'1F',	'+',	BSCH,	0
	DT	H'15',	H'09',	H'0F',	H'10',	H'1B',	H'1D',	RETCH,	0
	DT	H'0A',	H'0B',	H'0C',	':',	'@',	'~',	0,	0
	DT	H'0D',	'<',	'>',	'?',	SPCH,	0,	H'0E',	0
	ENDIF

; Character ROM
	ORG	H'1200'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'00',H'00',H'00'
	DT	H'0E',H'00',H'00',H'00',H'1E',H'1E',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'08',H'08',H'08'
	DT	H'00',H'18',H'66',H'00',H'08',H'00',H'1C',H'0C',H'30',H'0C',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'1C',H'18',H'3E',H'3E',H'30',H'7F',H'1C',H'7F',H'3E',H'3E',H'00',H'00',H'00',H'00',H'00',H'3E'
	DT	H'00',H'08',H'3F',H'3C',H'1F',H'7F',H'7F',H'3C',H'63',H'3C',H'78',H'67',H'0F',H'63',H'63',H'3E'
	DT	H'3F',H'3E',H'3F',H'3E',H'7E',H'63',H'63',H'63',H'63',H'66',H'7F',H'3C',H'00',H'3C',H'08',H'00'
	DT	H'18',H'00',H'07',H'00',H'38',H'00',H'38',H'00',H'07',H'18',H'60',H'07',H'1C',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'08',H'00',H'00',H'00',H'00',H'00',H'00',H'70',H'18',H'0E',H'00',H'3C'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'00',H'00',H'00'
	DT	H'0E',H'00',H'00',H'00',H'1E',H'1E',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'08',H'08',H'08'
	DT	H'00',H'18',H'66',H'00',H'08',H'00',H'1C',H'0C',H'30',H'0C',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'1C',H'18',H'3E',H'3E',H'30',H'7F',H'1C',H'7F',H'3E',H'3E',H'00',H'00',H'00',H'00',H'00',H'3E'
	DT	H'00',H'08',H'3F',H'3C',H'1F',H'7F',H'7F',H'3C',H'63',H'3C',H'78',H'67',H'0F',H'63',H'63',H'3E'
	DT	H'3F',H'3E',H'3F',H'3E',H'7E',H'63',H'63',H'63',H'63',H'66',H'7F',H'3C',H'00',H'3C',H'08',H'00'
	DT	H'18',H'00',H'07',H'00',H'38',H'00',H'38',H'00',H'07',H'18',H'60',H'07',H'1C',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'08',H'00',H'00',H'00',H'00',H'00',H'00',H'70',H'18',H'0E',H'00',H'3C'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'38',H'00',H'00',H'00'
	DT	H'1B',H'00',H'00',H'00',H'31',H'31',H'00',H'3C',H'1C',H'00',H'00',H'40',H'36',H'1C',H'1C',H'1C'
	DT	H'00',H'3C',H'66',H'36',H'3E',H'00',H'36',H'0C',H'18',H'18',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'36',H'1C',H'63',H'63',H'38',H'03',H'06',H'63',H'63',H'63',H'00',H'00',H'60',H'00',H'06',H'63'
	DT	H'3E',H'1C',H'66',H'66',H'36',H'66',H'66',H'66',H'63',H'18',H'30',H'66',H'06',H'77',H'67',H'63'
	DT	H'66',H'63',H'66',H'63',H'7E',H'63',H'63',H'63',H'63',H'66',H'63',H'0C',H'01',H'30',H'1C',H'00'
	DT	H'30',H'00',H'06',H'00',H'30',H'00',H'6C',H'00',H'06',H'18',H'60',H'06',H'18',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00',H'00',H'00',H'00',H'18',H'18',H'18',H'6E',H'66'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'38',H'00',H'00',H'00'
	DT	H'1B',H'00',H'00',H'00',H'31',H'31',H'00',H'3C',H'1C',H'00',H'00',H'40',H'36',H'1C',H'1C',H'1C'
	DT	H'00',H'3C',H'66',H'36',H'3E',H'00',H'36',H'0C',H'18',H'18',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'36',H'1C',H'63',H'63',H'38',H'03',H'06',H'63',H'63',H'63',H'00',H'00',H'60',H'00',H'06',H'63'
	DT	H'3E',H'1C',H'66',H'66',H'36',H'66',H'66',H'66',H'63',H'18',H'30',H'66',H'06',H'77',H'67',H'63'
	DT	H'66',H'63',H'66',H'63',H'7E',H'63',H'63',H'63',H'63',H'66',H'63',H'0C',H'01',H'30',H'1C',H'00'
	DT	H'30',H'00',H'06',H'00',H'30',H'00',H'6C',H'00',H'06',H'18',H'60',H'06',H'18',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00',H'00',H'00',H'00',H'18',H'18',H'18',H'6E',H'66'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'02',H'02',H'03'
	DT	H'11',H'08',H'60',H'00',H'30',H'30',H'08',H'0E',H'36',H'42',H'18',H'40',H'7F',H'1C',H'3E',H'3E'
	DT	H'00',H'3C',H'66',H'36',H'63',H'43',H'36',H'0C',H'0C',H'30',H'00',H'00',H'00',H'00',H'00',H'40'
	DT	H'63',H'1E',H'60',H'60',H'3C',H'03',H'03',H'60',H'63',H'63',H'18',H'18',H'30',H'00',H'0C',H'63'
	DT	H'63',H'36',H'66',H'43',H'66',H'46',H'46',H'43',H'63',H'18',H'30',H'66',H'06',H'7F',H'6F',H'63'
	DT	H'66',H'63',H'66',H'63',H'5A',H'63',H'63',H'63',H'36',H'66',H'61',H'0C',H'03',H'30',H'36',H'00'
	DT	H'00',H'00',H'06',H'00',H'30',H'00',H'4C',H'00',H'06',H'00',H'00',H'06',H'18',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00',H'00',H'00',H'00',H'18',H'18',H'18',H'3B',H'06'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'02',H'02',H'03'
	DT	H'11',H'08',H'60',H'00',H'30',H'30',H'08',H'0E',H'36',H'42',H'18',H'40',H'7F',H'1C',H'3E',H'3E'
	DT	H'00',H'3C',H'66',H'36',H'63',H'43',H'36',H'0C',H'0C',H'30',H'00',H'00',H'00',H'00',H'00',H'40'
	DT	H'63',H'1E',H'60',H'60',H'3C',H'03',H'03',H'60',H'63',H'63',H'18',H'18',H'30',H'00',H'0C',H'63'
	DT	H'63',H'36',H'66',H'43',H'66',H'46',H'46',H'43',H'63',H'18',H'30',H'66',H'06',H'7F',H'6F',H'63'
	DT	H'66',H'63',H'66',H'63',H'5A',H'63',H'63',H'63',H'36',H'66',H'61',H'0C',H'03',H'30',H'36',H'00'
	DT	H'00',H'00',H'06',H'00',H'30',H'00',H'4C',H'00',H'06',H'00',H'00',H'06',H'18',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00',H'00',H'00',H'00',H'18',H'18',H'18',H'3B',H'06'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'03',H'03',H'04'
	DT	H'1B',H'08',H'3E',H'66',H'18',H'1E',H'3E',H'06',H'63',H'24',H'18',H'60',H'7F',H'2A',H'3E',H'7F'
	DT	H'00',H'3C',H'24',H'7F',H'43',H'63',H'1C',H'06',H'0C',H'30',H'66',H'18',H'00',H'00',H'00',H'60'
	DT	H'63',H'18',H'30',H'60',H'36',H'03',H'03',H'60',H'63',H'63',H'18',H'18',H'18',H'7E',H'18',H'30'
	DT	H'63',H'63',H'66',H'03',H'66',H'16',H'16',H'03',H'63',H'18',H'30',H'36',H'06',H'7F',H'7F',H'63'
	DT	H'66',H'63',H'66',H'06',H'18',H'63',H'63',H'63',H'3E',H'66',H'30',H'0C',H'07',H'30',H'63',H'00'
	DT	H'00',H'1E',H'1E',H'3E',H'3C',H'3E',H'0C',H'6E',H'36',H'1C',H'70',H'66',H'18',H'37',H'3B',H'3E'
	DT	H'3B',H'6E',H'3B',H'3E',H'3F',H'33',H'63',H'63',H'63',H'63',H'7F',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'30',H'03',H'03',H'04'
	DT	H'1B',H'08',H'3E',H'66',H'18',H'1E',H'3E',H'06',H'63',H'24',H'18',H'60',H'7F',H'2A',H'3E',H'7F'
	DT	H'00',H'3C',H'24',H'7F',H'43',H'63',H'1C',H'06',H'0C',H'30',H'66',H'18',H'00',H'00',H'00',H'60'
	DT	H'63',H'18',H'30',H'60',H'36',H'03',H'03',H'60',H'63',H'63',H'18',H'18',H'18',H'7E',H'18',H'30'
	DT	H'63',H'63',H'66',H'03',H'66',H'16',H'16',H'03',H'63',H'18',H'30',H'36',H'06',H'7F',H'7F',H'63'
	DT	H'66',H'63',H'66',H'06',H'18',H'63',H'63',H'63',H'3E',H'66',H'30',H'0C',H'07',H'30',H'63',H'00'
	DT	H'00',H'1E',H'1E',H'3E',H'3C',H'3E',H'0C',H'6E',H'36',H'1C',H'70',H'66',H'18',H'37',H'3B',H'3E'
	DT	H'3B',H'6E',H'3B',H'3E',H'3F',H'33',H'63',H'63',H'63',H'63',H'7F',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'37',H'22',H'22',H'22'
	DT	H'0E',H'3E',H'25',H'66',H'0C',H'30',H'6B',H'1F',H'63',H'18',H'00',H'20',H'7F',H'7F',H'7F',H'7F'
	DT	H'00',H'18',H'00',H'36',H'1E',H'30',H'6E',H'00',H'0C',H'30',H'3C',H'18',H'00',H'00',H'00',H'30'
	DT	H'63',H'18',H'18',H'3C',H'33',H'3F',H'3F',H'30',H'3E',H'7E',H'00',H'00',H'0C',H'00',H'30',H'18'
	DT	H'7B',H'63',H'3E',H'03',H'66',H'1E',H'1E',H'03',H'7F',H'18',H'30',H'1E',H'06',H'6B',H'7B',H'63'
	DT	H'3E',H'63',H'3E',H'1C',H'18',H'63',H'63',H'6B',H'1C',H'3C',H'18',H'0C',H'0E',H'30',H'00',H'00'
	DT	H'00',H'30',H'36',H'63',H'36',H'63',H'1E',H'33',H'6E',H'18',H'60',H'36',H'18',H'7F',H'66',H'63'
	DT	H'66',H'33',H'6E',H'63',H'0C',H'33',H'63',H'63',H'36',H'63',H'33',H'0E',H'18',H'70',H'00',H'1F'
	MOVWF	PCL
	DT	H'08',H'08',H'00',H'08',H'08',H'00',H'08',H'00',H'08',H'00',H'08',H'37',H'22',H'22',H'22'
	DT	H'0E',H'3E',H'25',H'66',H'0C',H'30',H'6B',H'1F',H'63',H'18',H'00',H'20',H'7F',H'7F',H'7F',H'7F'
	DT	H'00',H'18',H'00',H'36',H'1E',H'30',H'6E',H'00',H'0C',H'30',H'3C',H'18',H'00',H'00',H'00',H'30'
	DT	H'63',H'18',H'18',H'3C',H'33',H'3F',H'3F',H'30',H'3E',H'7E',H'00',H'00',H'0C',H'00',H'30',H'18'
	DT	H'7B',H'63',H'3E',H'03',H'66',H'1E',H'1E',H'03',H'7F',H'18',H'30',H'1E',H'06',H'6B',H'7B',H'63'
	DT	H'3E',H'63',H'3E',H'1C',H'18',H'63',H'63',H'6B',H'1C',H'3C',H'18',H'0C',H'0E',H'30',H'00',H'00'
	DT	H'00',H'30',H'36',H'63',H'36',H'63',H'1E',H'33',H'6E',H'18',H'60',H'36',H'18',H'7F',H'66',H'63'
	DT	H'66',H'33',H'6E',H'63',H'0C',H'33',H'63',H'63',H'36',H'63',H'33',H'0E',H'18',H'70',H'00',H'1F'
	MOVWF	PCL
	DT	H'0F',H'F8',H'FF',H'FF',H'08',H'0F',H'0F',H'F8',H'F8',H'FF',H'FF',H'30',H'12',H'12',H'14'
	DT	H'00',H'08',H'24',H'66',H'06',H'31',H'0B',H'06',H'63',H'18',H'7E',H'30',H'3E',H'7F',H'7F',H'3E'
	DT	H'00',H'18',H'00',H'36',H'30',H'18',H'3B',H'00',H'0C',H'30',H'FF',H'7E',H'00',H'7F',H'00',H'18'
	DT	H'63',H'18',H'0C',H'60',H'7F',H'60',H'63',H'18',H'63',H'60',H'00',H'00',H'06',H'00',H'60',H'18'
	DT	H'7B',H'7F',H'66',H'03',H'66',H'16',H'16',H'7B',H'63',H'18',H'30',H'1E',H'06',H'63',H'73',H'63'
	DT	H'06',H'63',H'36',H'30',H'18',H'63',H'63',H'6B',H'1C',H'18',H'0C',H'0C',H'1C',H'30',H'00',H'00'
	DT	H'00',H'3E',H'66',H'03',H'33',H'7F',H'0C',H'33',H'66',H'18',H'60',H'1E',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'66',H'06',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'18',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'0F',H'F8',H'FF',H'FF',H'08',H'0F',H'0F',H'F8',H'F8',H'FF',H'FF',H'30',H'12',H'12',H'14'
	DT	H'00',H'08',H'24',H'66',H'06',H'31',H'0B',H'06',H'63',H'18',H'7E',H'30',H'3E',H'7F',H'7F',H'3E'
	DT	H'00',H'18',H'00',H'36',H'30',H'18',H'3B',H'00',H'0C',H'30',H'FF',H'7E',H'00',H'7F',H'00',H'18'
	DT	H'63',H'18',H'0C',H'60',H'7F',H'60',H'63',H'18',H'63',H'60',H'00',H'00',H'06',H'00',H'60',H'18'
	DT	H'7B',H'7F',H'66',H'03',H'66',H'16',H'16',H'7B',H'63',H'18',H'30',H'1E',H'06',H'63',H'73',H'63'
	DT	H'06',H'63',H'36',H'30',H'18',H'63',H'63',H'6B',H'1C',H'18',H'0C',H'0C',H'1C',H'30',H'00',H'00'
	DT	H'00',H'3E',H'66',H'03',H'33',H'7F',H'0C',H'33',H'66',H'18',H'60',H'1E',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'66',H'06',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'18',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'78',H'08',H'08',H'0B'
	DT	H'00',H'08',H'24',H'66',H'3F',H'1E',H'0B',H'1F',H'63',H'24',H'00',H'11',H'3E',H'6B',H'2A',H'1C'
	DT	H'00',H'18',H'00',H'36',H'61',H'0C',H'33',H'00',H'0C',H'30',H'3C',H'18',H'00',H'00',H'00',H'0C'
	DT	H'63',H'18',H'06',H'60',H'30',H'60',H'63',H'0C',H'63',H'60',H'00',H'00',H'0C',H'7E',H'30',H'18'
	DT	H'7B',H'63',H'66',H'03',H'66',H'06',H'06',H'63',H'63',H'18',H'33',H'36',H'06',H'63',H'63',H'63'
	DT	H'06',H'63',H'66',H'60',H'18',H'63',H'63',H'6B',H'3E',H'18',H'06',H'0C',H'38',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'03',H'33',H'03',H'0C',H'33',H'66',H'18',H'60',H'1E',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'1C',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'0C',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'78',H'08',H'08',H'0B'
	DT	H'00',H'08',H'24',H'66',H'3F',H'1E',H'0B',H'1F',H'63',H'24',H'00',H'11',H'3E',H'6B',H'2A',H'1C'
	DT	H'00',H'18',H'00',H'36',H'61',H'0C',H'33',H'00',H'0C',H'30',H'3C',H'18',H'00',H'00',H'00',H'0C'
	DT	H'63',H'18',H'06',H'60',H'30',H'60',H'63',H'0C',H'63',H'60',H'00',H'00',H'0C',H'7E',H'30',H'18'
	DT	H'7B',H'63',H'66',H'03',H'66',H'06',H'06',H'63',H'63',H'18',H'33',H'36',H'06',H'63',H'63',H'63'
	DT	H'06',H'63',H'66',H'60',H'18',H'63',H'63',H'6B',H'3E',H'18',H'06',H'0C',H'38',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'03',H'33',H'03',H'0C',H'33',H'66',H'18',H'60',H'1E',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'1C',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'0C',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'24',H'34',H'24'
	DT	H'00',H'00',H'24',H'66',H'00',H'00',H'6B',H'06',H'36',H'42',H'18',H'1A',H'1C',H'08',H'08',H'08'
	DT	H'00',H'00',H'00',H'7F',H'63',H'06',H'33',H'00',H'0C',H'30',H'66',H'18',H'18',H'00',H'00',H'06'
	DT	H'63',H'18',H'03',H'60',H'30',H'60',H'63',H'0C',H'63',H'60',H'18',H'18',H'18',H'00',H'18',H'00'
	DT	H'3B',H'63',H'66',H'43',H'66',H'46',H'06',H'63',H'63',H'18',H'33',H'66',H'46',H'63',H'63',H'63'
	DT	H'06',H'6B',H'66',H'63',H'18',H'63',H'36',H'7F',H'36',H'18',H'43',H'0C',H'70',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'03',H'33',H'03',H'0C',H'33',H'66',H'18',H'60',H'36',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'30',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'06',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'24',H'34',H'24'
	DT	H'00',H'00',H'24',H'66',H'00',H'00',H'6B',H'06',H'36',H'42',H'18',H'1A',H'1C',H'08',H'08',H'08'
	DT	H'00',H'00',H'00',H'7F',H'63',H'06',H'33',H'00',H'0C',H'30',H'66',H'18',H'18',H'00',H'00',H'06'
	DT	H'63',H'18',H'03',H'60',H'30',H'60',H'63',H'0C',H'63',H'60',H'18',H'18',H'18',H'00',H'18',H'00'
	DT	H'3B',H'63',H'66',H'43',H'66',H'46',H'06',H'63',H'63',H'18',H'33',H'66',H'46',H'63',H'63',H'63'
	DT	H'06',H'6B',H'66',H'63',H'18',H'63',H'36',H'7F',H'36',H'18',H'43',H'0C',H'70',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'03',H'33',H'03',H'0C',H'33',H'66',H'18',H'60',H'36',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'30',H'0C',H'33',H'63',H'6B',H'1C',H'63',H'06',H'18',H'18',H'18',H'00',H'06'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'32',H'4A',H'32'
	DT	H'00',H'3E',H'24',H'66',H'00',H'00',H'3E',H'0E',H'14',H'00',H'18',H'0C',H'08',H'1C',H'1C',H'00'
	DT	H'00',H'18',H'00',H'36',H'3E',H'63',H'33',H'00',H'18',H'18',H'00',H'00',H'18',H'00',H'18',H'03'
	DT	H'36',H'18',H'63',H'63',H'30',H'63',H'63',H'0C',H'63',H'30',H'18',H'18',H'30',H'00',H'0C',H'18'
	DT	H'03',H'63',H'66',H'66',H'36',H'66',H'06',H'66',H'63',H'18',H'33',H'66',H'66',H'63',H'63',H'63'
	DT	H'06',H'7B',H'66',H'63',H'18',H'63',H'1C',H'77',H'63',H'18',H'63',H'0C',H'60',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'63',H'33',H'63',H'0C',H'33',H'66',H'18',H'60',H'66',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'63',H'6C',H'33',H'36',H'7F',H'36',H'63',H'63',H'18',H'18',H'18',H'00',H'3F'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'32',H'4A',H'32'
	DT	H'00',H'3E',H'24',H'66',H'00',H'00',H'3E',H'0E',H'14',H'00',H'18',H'0C',H'08',H'1C',H'1C',H'00'
	DT	H'00',H'18',H'00',H'36',H'3E',H'63',H'33',H'00',H'18',H'18',H'00',H'00',H'18',H'00',H'18',H'03'
	DT	H'36',H'18',H'63',H'63',H'30',H'63',H'63',H'0C',H'63',H'30',H'18',H'18',H'30',H'00',H'0C',H'18'
	DT	H'03',H'63',H'66',H'66',H'36',H'66',H'06',H'66',H'63',H'18',H'33',H'66',H'66',H'63',H'63',H'63'
	DT	H'06',H'7B',H'66',H'63',H'18',H'63',H'1C',H'77',H'63',H'18',H'63',H'0C',H'60',H'30',H'00',H'00'
	DT	H'00',H'33',H'66',H'63',H'33',H'63',H'0C',H'33',H'66',H'18',H'60',H'66',H'18',H'6B',H'66',H'63'
	DT	H'66',H'33',H'06',H'63',H'6C',H'33',H'36',H'7F',H'36',H'63',H'63',H'18',H'18',H'18',H'00',H'3F'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'29',H'21',H'29'
	DT	H'00',H'00',H'22',H'3E',H'00',H'00',H'08',H'3C',H'77',H'00',H'00',H'08',H'00',H'3E',H'3E',H'00'
	DT	H'00',H'18',H'00',H'36',H'08',H'61',H'6E',H'00',H'30',H'0C',H'00',H'00',H'18',H'00',H'18',H'01'
	DT	H'1C',H'7E',H'7F',H'3E',H'78',H'3E',H'3E',H'0C',H'3E',H'1E',H'00',H'0C',H'60',H'00',H'06',H'18'
	DT	H'3E',H'63',H'3F',H'3C',H'1F',H'7F',H'0F',H'5C',H'63',H'3C',H'1E',H'67',H'7F',H'63',H'63',H'3E'
	DT	H'0F',H'3E',H'67',H'3E',H'3C',H'3E',H'08',H'36',H'63',H'3C',H'7F',H'3C',H'40',H'3C',H'00',H'00'
	DT	H'00',H'6E',H'3E',H'3E',H'6E',H'3E',H'1E',H'3E',H'67',H'3C',H'60',H'67',H'3C',H'63',H'66',H'3E'
	DT	H'3E',H'3E',H'0F',H'3E',H'38',H'6E',H'1C',H'36',H'63',H'7E',H'7F',H'70',H'18',H'0E',H'00',H'73'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'29',H'21',H'29'
	DT	H'00',H'00',H'22',H'3E',H'00',H'00',H'08',H'3C',H'77',H'00',H'00',H'08',H'00',H'3E',H'3E',H'00'
	DT	H'00',H'18',H'00',H'36',H'08',H'61',H'6E',H'00',H'30',H'0C',H'00',H'00',H'18',H'00',H'18',H'01'
	DT	H'1C',H'7E',H'7F',H'3E',H'78',H'3E',H'3E',H'0C',H'3E',H'1E',H'00',H'0C',H'60',H'00',H'06',H'18'
	DT	H'3E',H'63',H'3F',H'3C',H'1F',H'7F',H'0F',H'5C',H'63',H'3C',H'1E',H'67',H'7F',H'63',H'63',H'3E'
	DT	H'0F',H'3E',H'67',H'3E',H'3C',H'3E',H'08',H'36',H'63',H'3C',H'7F',H'3C',H'40',H'3C',H'00',H'00'
	DT	H'00',H'6E',H'3E',H'3E',H'6E',H'3E',H'1E',H'3E',H'67',H'3C',H'60',H'67',H'3C',H'63',H'66',H'3E'
	DT	H'3E',H'3E',H'0F',H'3E',H'38',H'6E',H'1C',H'36',H'63',H'7E',H'7F',H'70',H'18',H'0E',H'00',H'73'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'78',H'10',H'78'
	DT	H'00',H'00',H'00',H'06',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'30',H'00',H'00',H'66',H'00',H'00',H'00',H'00',H'00'
	DT	H'06',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'60',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'78',H'10',H'78'
	DT	H'00',H'00',H'00',H'06',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'0C',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'30',H'00',H'00',H'66',H'00',H'00',H'00',H'00',H'00'
	DT	H'06',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'60',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'20',H'78',H'20'
	DT	H'00',H'00',H'00',H'06',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'70',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'FF'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'33',H'00',H'00',H'66',H'00',H'00',H'00',H'00',H'00'
	DT	H'06',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'30',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'20',H'78',H'20'
	DT	H'00',H'00',H'00',H'06',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'70',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'FF'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'33',H'00',H'00',H'66',H'00',H'00',H'00',H'00',H'00'
	DT	H'06',H'30',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'30',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'03',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'1E',H'00',H'00',H'3C',H'00',H'00',H'00',H'00',H'00'
	DT	H'0F',H'78',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'1F',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'03',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'1E',H'00',H'00',H'3C',H'00',H'00',H'00',H'00',H'00'
	DT	H'0F',H'78',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'1F',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	MOVWF	PCL
	DT	H'00',H'00',H'00',H'00',H'08',H'08',H'08',H'08',H'08',H'08',H'08',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'
	DT	H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00',H'00'

	END
